﻿using System;

namespace Fast_CS_CRC_Generator {
   class Program {
      static readonly byte[] testData = new byte[] { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39 }; //"123456789";
      static readonly byte[] testData1 = new byte[] { 0xE4, 0x6D, 0x46, 0xF8, 0xFB, 0x16, 0xAB, 0x52, 0xED, 0x64, 0x50, 0x23, 0x63, 0x0D, 0x80, 0xBC, 0x98, 0xD8, 0x16, 0xFF, 0xEF, 0x3C, 0x4E, 0xD9, 0xBD, 0xA1, 0x85, 0x95, 0x13 };
      static readonly byte[] testData2 = new byte[] { 0x42, 0xDF, 0x54, 0x99, 0x99, 0x20, 0x98, 0xCF, 0xF1, 0xFC, 0xF5, 0x9E, 0xED, 0x36, 0x61, 0x4A, 0xA2, 0x31, 0xF3, 0x3E, 0x02, 0xB1, 0x8A, 0x95, 0xE5, 0xF7, 0x88 };
      static readonly byte[] testData3 = new byte[] { 0x5B, 0xB5, 0x06, 0x04, 0x52, 0xDE, 0xD8, 0xEC, 0x13, 0x35, 0x6B, 0x59, 0xC8, 0x66, 0xA4, 0xF6, 0xC9, 0xA8, 0x63, 0x03, 0xB9, 0xA8, 0xFD, 0xD9, 0x57 };

      static readonly CRC8GeneratorLookUp crc8Generator = new();
      static readonly CRC16GeneratorLookUp crc16Generator = new();
      static readonly CRC32GeneratorLookUp crc32Generator = new();

      static uint stream8TestData(byte[] data) {
         for (int i = 0; i < data.Length; i++)
            crc8Generator.streamCRC(data[i]);
         return crc8Generator.streamFin();
      }

      static uint stream16TestData(byte[] data) {
         for (int i = 0; i < data.Length; i++)
            crc16Generator.streamCRC(data[i]);
         return crc16Generator.streamFin();
      }

      static uint stream32TestData(byte[] data) {
         for (int i = 0; i < data.Length; i++)
            crc32Generator.streamCRC(data[i]);
         return crc32Generator.streamFin();
      }

      static void Main(string[] args) {
         Console.WriteLine("C# CRC-Generator Test");

         Console.WriteLine("==========================\n");
         Console.WriteLine("  CRC8");
         Console.WriteLine("==========================");

         Console.WriteLine("  genCRC: " + crc8Generator.generateCRC(testData));
         Console.WriteLine("  stream: " + stream8TestData(testData));
         Console.WriteLine("--------------------------");
         Console.WriteLine("1 genCRC: " + crc8Generator.generateCRC(testData1));
         Console.WriteLine("  stream: " + stream8TestData(testData1));
         Console.WriteLine("--------------------------");
         Console.WriteLine("2 genCRC: " + crc8Generator.generateCRC(testData2));
         Console.WriteLine("  stream: " + stream8TestData(testData2));
         Console.WriteLine("--------------------------");
         Console.WriteLine("3 genCRC: " + crc8Generator.generateCRC(testData3));
         Console.WriteLine("  stream: " + stream8TestData(testData3));


         Console.WriteLine("==========================\n");
         Console.WriteLine("  CRC16");
         Console.WriteLine("==========================");

         Console.WriteLine("  genCRC: " + crc16Generator.generateCRC(testData));
         Console.WriteLine("  stream: " + stream16TestData(testData));
         Console.WriteLine("--------------------------");
         Console.WriteLine("1 genCRC: " + crc16Generator.generateCRC(testData1));
         Console.WriteLine("  stream: " + stream16TestData(testData1));
         Console.WriteLine("--------------------------");
         Console.WriteLine("2 genCRC: " + crc16Generator.generateCRC(testData2));
         Console.WriteLine("  stream: " + stream16TestData(testData2));
         Console.WriteLine("--------------------------");
         Console.WriteLine("3 genCRC: " + crc16Generator.generateCRC(testData3));
         Console.WriteLine("  stream: " + stream16TestData(testData3));

         Console.WriteLine("==========================");
         Console.WriteLine("  CRC32");
         Console.WriteLine("==========================");

         Console.WriteLine("  genCRC: " + crc32Generator.generateCRC(testData));
         Console.WriteLine("  stream: " + stream32TestData(testData));
         Console.WriteLine("--------------------------");
         Console.WriteLine("1 genCRC: " + crc32Generator.generateCRC(testData1));
         Console.WriteLine("  stream: " + stream32TestData(testData1));
         Console.WriteLine("--------------------------");
         Console.WriteLine("2 genCRC: " + crc32Generator.generateCRC(testData2));
         Console.WriteLine("  stream: " + stream32TestData(testData2));
         Console.WriteLine("--------------------------");
         Console.WriteLine("3 genCRC: " + crc32Generator.generateCRC(testData3));
         Console.WriteLine("  stream: " + stream32TestData(testData3));
      }
   }
}

/* Output ist:
 
C# CRC-Generator Test
==========================

  CRC8
==========================
  genCRC: 244
  stream: 244
--------------------------
1 genCRC: 133
  stream: 133
--------------------------
2 genCRC: 233
  stream: 233
--------------------------
3 genCRC: 229
  stream: 229
==========================

  CRC16
==========================
  genCRC: 58828
  stream: 58828
--------------------------
1 genCRC: 39760
  stream: 39760
--------------------------
2 genCRC: 59077
  stream: 59077
--------------------------
3 genCRC: 32278
  stream: 32278
==========================
  CRC32
==========================
  genCRC: 3421780262
  stream: 3421780262
--------------------------
1 genCRC: 1628163719
  stream: 1628163719
--------------------------
2 genCRC: 2942970280
  stream: 2942970280
--------------------------
3 genCRC: 948803831
  stream: 948803831
 
 */